@@ -3,16 +3,17 @@
 """
 from . import helpers, utils, wrappers
 from .conflicts import ConflictResolution
-from .helpers import (MutableField, Serializable, SimpleHelpFormatter, choice,
-                      field, list_field, mutable_field, subparsers)
-from .parsing import ArgumentParser
+from .help_formatter import SimpleHelpFormatter
+from .helpers import (MutableField, Serializable, choice, field, flag,
+                      list_field, mutable_field, subparsers)
+from .parsing import ArgumentParser, ParsingError
 from .utils import InconsistentArgumentError
 
 __all__ = [
     "helpers", "utils", "wrappers",
     "ConflictResolution",
     "MutableField", "Serializable", "SimpleHelpFormatter", "choice",
-    "field", "list_field", "mutable_field", "subparsers",
+    "field", "flag", "list_field", "mutable_field", "subparsers",
     "ArgumentParser",
     "InconsistentArgumentError",
 ]
