@@ -8,10 +8,12 @@ from typing import *
 from typing import cast
 
 from .. import docstring, utils
-from ..utils import Dataclass, DataclassType
 from ..helpers import dict_field
-from .wrapper import Wrapper
 from ..logging_utils import get_logger
+from ..utils import Dataclass, DataclassType
+from .field_parsing import get_parsing_fn
+from .field_metavar import get_metavar
+from .wrapper import Wrapper
 
 logger = get_logger(__file__)
 
@@ -38,6 +40,8 @@ class FieldWrapper(Wrapper[dataclasses.Field]):
     # TODO: This can often make "--help" messages a bit crowded
     add_dash_variants: ClassVar[bool] = False
 
+    # Wether to add the `dest` to the list of option strings.
+    add_dest_to_option_strings: ClassVar[bool] = True
 
     def __init__(self, field: dataclasses.Field, parent: Any = None, prefix: str=""):
         super().__init__(wrapped=field, name=field.name)
@@ -138,20 +142,40 @@ class FieldWrapper(Wrapper[dataclasses.Field]):
                          f"'{attribute}'")
 
     def get_arg_options(self) -> Dict[str, Any]:
+        """ Create the `parser.add_arguments` kwargs for this field. """
         if not self.field.init:
+            # Don't add command-line arguments for fields that have `init=False`.
+            # TODO: See (https://github.com/lebrice/SimpleParsing/issues/20#issue-619198492)
             return {}
 
         _arg_options: Dict[str, Any] = {}
         # TODO: should we explicitly use `str` whenever the type isn't a builtin
         # type? or try to use it as a constructor?
-        _arg_options["type"] = self.type
+        # TODO: [Improvement] @lebrice Maybe overhaul the 'type' function, in a
+        # similar way as with is done with fields in the Serializable class. 
+        _arg_options["type"] = get_parsing_fn(self.type)
         _arg_options["help"] = self.help
         _arg_options["required"] = self.required
         _arg_options["dest"] = self.dest
         _arg_options["default"] = self.default
+        _arg_options["metavar"] = get_metavar(self.type)
+        
+        if self.is_choice:
+            _arg_options["type"] = str
+            _arg_options["choices"] = list(self.choices)
+            if self.is_list:
+                _arg_options["nargs"] = argparse.ZERO_OR_MORE
+            # We use the default 'metavar' generated by argparse.
+            _arg_options.pop("metavar", None)
+        
+        elif self.is_optional:
+            _arg_options["type"] = get_parsing_fn(self.field.type)
 
+        elif self.is_union:
+            logger.debug(f"Parsing a Union type!")
+            _arg_options["type"] = get_parsing_fn(self.field.type)
 
-        if self.is_enum:
+        elif self.is_enum:
             logger.debug(f"Adding an Enum attribute '{self.name}'")
             # we actually parse enums as string, and convert them back to enums
             # in the `process` method.
@@ -168,31 +192,23 @@ class FieldWrapper(Wrapper[dataclasses.Field]):
                     _arg_options["default"] = enum_to_str(self.default)
 
         elif self.is_list:
-            # Check if typing.List or typing.Tuple was used as an annotation, in
-            # which case we can automatically convert items to the desired item
-            # type.
-            T = utils.get_argparse_type_for_container(self.type)
-            logger.debug(
-                f"Adding a List attribute '{self.name}'"
-                f"with items of type '{T}'"
-            )
+            logger.debug(f"Adding a List attribute '{self.name}': {self.type}")
             _arg_options["nargs"] = "*"
-            _arg_options["type"] = T
 
             if self.is_reused:
+                # TODO: Only the 'single-level' lists (not reused) use the new
+                # `get_parsing_fn` function (for now).  
                 type_fn = utils._parse_multiple_containers(self.type)
                 type_fn.__name__ = utils.get_type_name(self.type)
                 _arg_options["type"] = type_fn
 
         elif self.is_tuple:
-            T = utils.get_argparse_type_for_container(self.type)
-            logger.debug(
-                f"Adding a Tuple attribute '{self.name}' "
-                f"with items of type '{T}'"
-            )
+            logger.debug(f"Adding a Tuple attribute '{self.name}' with type {self.type}")
             _arg_options["nargs"] = utils.get_container_nargs(self.type)
-            _arg_options["type"] = utils._parse_container(self.type)
-            
+            _arg_options["type"] = get_parsing_fn(self.field.type)
+            # No need atm, since the HelpFormatter takes care of creating the
+            # metavar.
+            # _arg_options["type"].__name__ = utils.get_type_name(_arg_options["type"])
 
             if self.is_reused:
                 type_fn = utils._parse_multiple_containers(self.type)
@@ -202,8 +218,17 @@ class FieldWrapper(Wrapper[dataclasses.Field]):
         elif self.is_bool:
             _arg_options["type"] = utils.str2bool
             _arg_options["type"].__name__ = "bool"
-            # if self.default is not None:
             _arg_options["nargs"] = "?"
+        
+        else:
+            # "Plain" / simple argument.
+            # For the metavar, use a custom passed value, if present, else do
+            # not put any value in (which uses the usual value from argparse).
+            if self.metavar:
+                _arg_options["metavar"] = self.metavar
+            else:
+                # Remove the 'metavar' that we auto-generated above.
+                _arg_options.pop("metavar", None)
 
         if self.is_reused:
             if self.required:
@@ -316,11 +341,12 @@ class FieldWrapper(Wrapper[dataclasses.Field]):
             return raw_parsed_value
 
         elif self.type not in utils.builtin_types:
+            # TODO: what if we actually got an auto-generated parsing function?
             try:
                 # if the field has a weird type, we try to call it directly.
                 return self.type(raw_parsed_value)
             except Exception as e:
-                logger.warning(
+                logger.debug(
                     f"Unable to instantiate the field '{self.name}' of type "
                     f"'{self.type}' by using the type as a constructor. "
                     f"Returning the raw parsed value instead "
@@ -432,6 +458,11 @@ class FieldWrapper(Wrapper[dataclasses.Field]):
             ]
             options.extend(additional_options)
             dashes.extend(additional_dashes)
+        
+        if type(self).add_dest_to_option_strings:
+            dashes.append("-" if len(self.dest) == 1 else "--")
+            options.append(self.dest)
+        
         # remove duplicates by creating a set.
         option_strings = set(
             f"{dash}{option}" for dash, option in zip(dashes, options)
@@ -524,10 +555,17 @@ class FieldWrapper(Wrapper[dataclasses.Field]):
             # if the dataclass holding this field has a default value (either
             # when passed  manually or by nesting), use the corresponding
             # attribute on that default instance.
+            
+            # TODO: When that default value is 'None' (even for a dataclass),
+            # then we need to.. ?
+
             defaults = []
             for default_dataclass_instance in self.parent.defaults:
-                parent_value = getattr(default_dataclass_instance, self.name)
-                defaults.append(parent_value)
+                if default_dataclass_instance is None:
+                    default_value = default
+                else:
+                    default_value = getattr(default_dataclass_instance, self.name)
+                defaults.append(default_value)
             default = defaults[0] if len(defaults) == 1 else defaults
 
         if self.is_reused and default is not None:
@@ -586,17 +624,6 @@ class FieldWrapper(Wrapper[dataclasses.Field]):
     def type(self) -> Type[Any]:
         if self._type is not None:
             return self._type
-        elif utils.is_optional(self.field.type):
-            type_args = set(utils.get_type_arguments(self.field.type))
-            # TODO: What do we do if the type is something like Union[str, int, float]?
-            if str in type_args:
-                self._type = str
-            else:
-                type_args.remove(type(None))
-                # get the first non-NoneType type argument.
-                self._type = type_args.pop()
-        
-        
         else:
             self._type = self.field.type
         return self._type
@@ -609,14 +636,21 @@ class FieldWrapper(Wrapper[dataclasses.Field]):
         return self.choices is not None
 
     @property
-    def choices(self):
-        choices = self.custom_arg_options.get("choices", None)
-        if choices is None:
+    def choices(self) -> Optional[List]:
+        choices = self.custom_arg_options.get("choices")
+        choice_dict = self.choice_dict
+        if not choices:
             return None
         if len(choices) == 1 and isinstance(choices[0], dict):
-            return choices[0]
+            choice_dict = choices[0]
+            assert False, "HERE"
+            return choice_dict
         return choices
 
+    @property
+    def choice_dict(self) -> Optional[Dict]:
+        return self.field.metadata.get("choice_dict")
+
     @property
     def help(self) -> Optional[str]:
         if self._help:
@@ -645,6 +679,12 @@ class FieldWrapper(Wrapper[dataclasses.Field]):
 
     @property
     def metavar(self) -> Optional[str]:
+        """ Returns the 'metavar' when set using one of the `field` functions,
+        else None. 
+        """
+        if self._metavar:
+            return self._metavar
+        self._metavar = self.custom_arg_options.get("metavar")
         return self._metavar
 
     @metavar.setter
@@ -675,6 +715,10 @@ class FieldWrapper(Wrapper[dataclasses.Field]):
     def is_optional(self) -> bool:
         return utils.is_optional(self.field.type)
 
+    @property
+    def is_union(self) -> bool:
+        return utils.is_union(self.field.type)
+
     @property
     def is_subparser(self) -> bool:
         return utils.is_subparser_field(self.field)
@@ -688,14 +732,20 @@ class FieldWrapper(Wrapper[dataclasses.Field]):
         return self._parent
 
     @property
-    def subparsers_dict(self) -> Dict[str, Type]:
-        if "subparsers" in self.field.metadata:
+    def subparsers_dict(self) -> Optional[Dict[str, Type]]:
+        """ The dict of subparsers, which is created either when using a
+        Union[<dataclass_1>, <dataclass_2>] type annotation, or when using the
+        `subparsers()` function.
+        """
+        if self.field.metadata.get("subparsers"):
             return self.field.metadata["subparsers"]
-        else:
-            type_arguments = utils.get_type_arguments(self.type)
-            return {
-                dataclass_type.__name__.lower(): dataclass_type for dataclass_type in type_arguments
-            }
+        elif self.is_union:
+            type_arguments = utils.get_type_arguments(self.field.type)
+            if type_arguments and any(map(utils.is_dataclass_type, type_arguments)):    
+                return {
+                    utils.get_type_name(dataclass_type).lower(): dataclass_type
+                    for dataclass_type in type_arguments
+                }
 
     def add_subparsers(self, parser: argparse.ArgumentParser):
         assert self.is_subparser
